#include "SanguruButton.h"
#include "Arduino.h"

const byte buttonNone = 0; 
const byte onReleased = 1;
const byte onPressed = 2;
const byte released = 4;
const byte pressed = 8;
const byte repeat = 16;
const byte longPressed = 32;

// constructor 앞에는 void 조차도 없다.
SanguruButton::SanguruButton(byte buttonPin) { 

  	this->buttonPin = buttonPin;
  	pinMode(buttonPin,INPUT_PULLUP);

}

void SanguruButton::update() {

	unsigned long thisMil = millis();
	if (debounceMode == ON_DEBOUNCE){
		if(thisMil > debounceEndMil ) {
			debounceMode = OVER_DEBOUNCE;
		}
		else {
			return;
		} 
	}

	if (isButtonChanged == false) {
		buttonRead = digitalRead(buttonPin);
		if (prevButtonRead != buttonRead)  {
			debounceMode = ON_DEBOUNCE;
			debounceEndMil = thisMil + debounceDurationMil; 
			buttonPressMil = thisMil;
			isButtonChanged = true;		
		}		
	}
}

byte SanguruButton::getButton () {

	unsigned long thisMil = millis();

    if (isButtonChanged == true) {
		if (prevButtonRead == HIGH && buttonRead == LOW  ) {
			buttonOperation = pressed;
		}
		else if (prevButtonRead == LOW && buttonRead == HIGH  ) {
			
			if (isSetLongPress && isLongPressOn ) {
				buttonOperation = onReleased;  // ignore released
				isLongPressOn = false;
			}
			else 
				buttonOperation = released;
			
		}
		
		prevButtonRead = buttonRead;
		buttonPressMil = thisMil;
		isButtonChanged = false;
	}
	else if (prevButtonRead == HIGH && buttonRead == HIGH  ) {
		buttonOperation = onReleased;
	}
	else if (prevButtonRead == LOW &&  buttonRead == LOW ) {
		if ( isSetRepeat &&(prevButtonOperation == onPressed || prevButtonOperation == repeat) ) {
		 	if ( thisMil- buttonPressMil > repeatDurationMil && repeatDurationMil > 0 ) {
	    		buttonOperation = repeat;
	    		buttonPressMil = thisMil;
	    	}
		    else 
		        buttonOperation = onPressed;
		}
		else if ( isSetLongPress ) {
			if ( isSetLongPress && (thisMil- buttonPressMil > longPressDurationMil && longPressDurationMil > 0 )){
	    		buttonOperation = longPressed;
		    	buttonPressMil = thisMil;
		    	isLongPressOn = true;
			}
			else {
				buttonOperation = onPressed;
			}
	    }
	    else {
	    	buttonOperation = onPressed;
	    }        
	}

	prevButtonOperation = buttonOperation;

  	return buttonOperation;
}

void SanguruButton::setLongPress(bool trueOrFalse) {

	// isSetLongPress와 isSetRepeat는 하나가 true로 되면 나머지는 false로 된다 
	isSetLongPress = trueOrFalse;
	if (trueOrFalse == true)
		isSetRepeat = false;
}

void SanguruButton::setRepeat(bool trueOrFalse) {
	
	// isSetLongPress와 isSetRepeat는 하나가 true로 되면 나머지는 false로 된다 
	isSetRepeat = trueOrFalse;
	if (trueOrFalse == true)
		isSetLongPress = false;

}